/*
Paolo Gibellini
2009-05-25, from sratch

Significative steps:
- golden eggs instead of homes
- less ior for the water, or we cannot see the houses
- a very different way.... an underwater IRTC? I can recycle my old sea wip...
- random brick rotation
- grouped objects and rotated union
- object placement over text shape, with ground quote

Deprecated:
- Homes instead of bricks? Could be nice a irtc-city
- landscape: a (bad) traced island

Ideas:
- a Boat with a treasure finder?
- significative colors and lights
- a better underwater

Deprecated Ideas:
- Add a little truck with the briks? Or a horse-trained vecture? Or little workers?
- people on bricks over the sea (non-fish version)?
- some people?

Thanks:
Mike (http://povray.tashcorp.net/tutorials/qd_sierpinski/) for pyramid
F_loemuller for his egg
Some people for the water

*/

#include "functions.inc"
#include "textures.inc"
#include "metals.inc"
#include "golds.inc"
#include "./_shapes_lo.inc"

// Parameters ---------------------
#declare is_night=1;            // Radiosity on
#declare with_photons=1;        // use photons (for caustics)
#declare show_bricks=1;         // "bricks" traced over a text but with the elevation of a ground         
#declare show_island=1;         // show base traced for the elevation of the bricks
#declare show_irtc=0;           // show the text traced
#declare show_fishes=1;         // show the fishes
#declare show_water=1;          // show water surface
#declare visible_water=1;       // kind of texture for the water
#declare three_big_fishes=0;    // three big fishes or a pack?
#declare fishes_number=200;     // number of fishes       
#declare golden_sand=0;         // brown (0) or golden (1) sand?
#declare blurred_water=1;       // blurred water (slow but nice)?
#declare blur_amount=4;         // How many rays to shoot  // 4 ok // 5 prettier but slower //40 for me is impossible

#declare show_moon=0;           // to be implemented
#declare show_floor=0;          // old ground (deprecated)
#declare show_sea=0;            // old sea (deprecated)

// --------------------------------


// Global settings ------------------------------------

// radiosity (global illumination) settings
global_settings {
	#if(is_night)
	  	radiosity {
		    pretrace_start 0.08           // start pretrace at this size
		    pretrace_end   0.04           // end pretrace at this size
		    count 35                      // higher -> higher quality (1..1600) [35]
		    nearest_count 5               // higher -> higher quality (1..10) [5]
		    error_bound 1.8               // higher -> smoother, less accurate [1.8]
		    recursion_limit 3             // how much interreflections are calculated (1..5+) [3]
		    low_error_factor .5           // reduce error_bound during last pretrace step
		    gray_threshold 0.0            // increase for weakening colors (0..1) [0]
		    minimum_reuse 0.015           // reuse of old radiosity samples [0.015]
		    brightness 1                  // brightness of radiosity effects (0..1) [1]
		
		    adc_bailout 0.01/2
		    //normal on                   // take surface normals into account [off]
		    //media on                    // take media into account [off]
		    //save_file "radiosity_26.file"       // save radiosity data
		    load_file "radiosity_26.file"       // load saved radiosity data
		    //always_sample off           // turn sampling in final trace off [on]
		    //max_sample 1.0              // maximum brightness of samples
	  	}
	#else
		assumed_gamma 1.0
	#end
	#if(with_photons)
	  	photons {
	      	spacing 0.005 //0.003 better but slow -- 0.01
	     	//save_file "photons_26.file"
	     	load_file "photons_26.file"
	    }
	    max_trace_level 10
    #end
}


// Camera & lights -------------
#declare camera_look_at = <0,0,0> ;
#declare camera_location = <0,10.5,-14>;
//#declare camera_location = <0,50,0>; // Flight view
//#declare camera_location = <0,0,-40>; // Front

camera {
//	orthographic
	location camera_location
	angle 40
	look_at camera_look_at
}

// Light ----------------------------------
#if(!is_night)
	light_source {
		<0, 0, 0>
		color rgb <1, 1, 1>
		translate <1500,2500,-2500>
		#if(with_photons)
			photons {
			    reflection off
			    refraction on
		 	}
		#end		 
	}
	light_source{<0,2,2> color rgb <1, 1, 1>}
#else
	light_source {
		<0, 0, 0>
		color rgb <255, 215, 0>/255
		translate <1500,2500,-2500>
		/*
		#if(with_photons)
			photons {
			    reflection off
			    refraction on
		 	}
		#end		 
		*/
	}
	//light_source{<0,2,2> color rgb <0.2, 0.2, 0.4>}
	light_source{<0,2,2> color rgb <100, 149, 237>/255}
#end
// ----------------------------------------

// Textures -----------------

// Trees trunk texture
#declare t_trunk=texture{
	pigment {
		marble
		color_map {
			[0.0 rgb <0.737, 0.585, 0.415>]
			[0.2 rgb <0.737, 0.585, 0.415>*0.2]
			[0.9 rgb <0.737, 0.585, 0.415>*0.9]
		}
		turbulence 0.2
		octaves 4
	}
	normal {
		marble
		turbulence 0.2
		octaves 4
	}
	scale 0.03
}

// Trees leaves texture
#declare t_leaves=texture {      
	pigment {
		agate
		color_map {
			[ 0.0     rgbt <0.3663, 0.5682, 0.0687, 1.0> ]
			[ 0.25    rgbt <0.3663, 0.5682, 0.0687, 0.82> ]
			[ 0.31    rgbt <0.0887, 0.9886, 0.4438, 0.01> ]
			[ 0.39    rgbt <0.1308, 0.9083, 0.5524, 0.2> ]
			[ 0.80    rgbt <0.1755, 0.5095, 0.448, 0.43> ]
			[ 1.0     rgbt <0.1755, 0.5095, 0.448, 1.0> ]
		}
		turbulence 0.6
		ramp_wave
	}
	normal {
		agate
		turbulence 0.6
		ramp_wave
	}      
	finish {
		ambient 0.1
		diffuse 0.5
		phong_size 81.0
		roughness 0.0
	}
	// scale 2.1 // mmm... to be tested
	scale 0.3
}

// Leaves material
#declare m_foglie=material{
	texture{t_leaves}
	interior {
		
	}
}

#declare t_3=texture {
	pigment {
      average pigment_map {
        [1.0  rgb <1,0.8,0.8>]
        [2.0  rgb <1,1,1>]
      }
    }
	normal{
		average normal_map{
        [1.0  bumps 0.1]
        [2.0  bumps 0.4 rotate 60 scale .015]
        [4.0  bumps 0.2 rotate 120 scale .005]
		}
	}
  	#if (!is_night)
		finish{diffuse 1.3 phong .1}
	#end
}

#declare t_sabbia = texture {
     pigment {
        granite
        color_map {
           [ 0.0     rgbft <0.670588, 0.505882, 0.321569, 0.0, 0.0> ]
           [ 1.0     rgbft <0.941176, 0.878431, 0.792157, 0.0, 0.0> ]
        }
        turbulence 0.75
        ramp_wave
        //scale 0.1 // Too big at high resolutions
        scale  0.01
     }
}   
//#declare t_brick=texture {pigment{color rgb x}};
#declare t_brick=texture {
	pigment {
		granite
		color_map {
			[0.0 rgbft <0.7,0.7,0.7,0,0>]
			[0.2 rgbft <0.7,0.7,0.7,0,0>]
			[0.3 rgb <0.9, 0.8, 0.7>]
			[0.5 rgbft <1,1,1,0,0>]
			[1.0 rgbft <1,1,1,0,0>]
		}
		turbulence 0.5
		scale 1.2
	}
	normal {
		granite
		turbulence 0.5
	    slope_map {
	        [ 0.0    <1,0> ]
	        [ 0.6    <1,0> ]
	        [ 0.6    <1,-1> ]
	        [ 1.0    <0,0> ]
	    }
		scale 1.2
	}
	scale 0.01 	
}

#declare t_brick=t_sabbia;

#declare t_sea=texture {
	pigment{
		bozo
		turbulence 0.76
		color_map { 
			[0.3  rgb<0.1,0.3,0.9> ]
			[0.5  rgbf<0.4,0.7,0.8,1> ]
			[0.6  rgb <0.3,0.6,0.9>     ]
			[0.65 rgb <0.4,0.7,0.99>     ]
			[1.0  rgb <0.2,0.5,0.8>     ] 
		}
	}
	pigment{
		color rgbf <0.2,0.8,0.9,0.6>
	}
}

#declare m_water = color_map {
	[ 0.00 rgbt <0.217,0.552,0.877, 0.9> ]
	[ 0.90 rgbt <0.137,0.437,0.697, 0.95> ]
	[ 1.00 rgb <1,1,1> ]
}
#declare t_water = texture {
    pigment{
    	rgbt <0.217,0.552,0.877, 0.9>
    	/*
    	gradient z 
    	color_map {m_water}
	    turbulence 0.9
	    */
    }
    normal {
        ripples 0.75
        frequency 1
        translate <-3,0,3>
    }
    finish {
		diffuse 0.3
		ambient 0.0
        reflection {0.3, 1 fresnel}
        conserve_energy
		specular 0.3
		roughness 0.003
    }
}
#declare t_sea=t_water;

// Textures -------------------------------
#declare t_sabbiosa=texture {
     pigment {
        granite
        color_map {
           [ 0.0     rgbft <0.670588, 0.505882, 0.321569, 0.0, 0.0> ]
           [ 1.0     rgbft <0.941176, 0.878431, 0.792157, 0.0, 0.0> ]
        }
        turbulence 0.75
        ramp_wave
        //scale 0.1 // Troppo grandi per alte risoluzioni
        scale  0.01
     }
}
#declare t_sabbia_oro=texture {
     pigment {
        granite
        color_map {
           [ 0.0     rgbft <1, 0.827, 0.141, 0.0, 0.0> ]
           //[ 1.0     rgbft <0.670588, 0.505882, 0.321569, 0.0, 0.0> ]
           [ 1.0     rgbft <0.470588, 0.305882, 0.121569, 0.0, 0.0> ]
        }
        turbulence 0.75
        ramp_wave
        scale  0.01
     }
}

#if(golden_sand)
	#declare t_sabbia=t_sabbia_oro;
#else
	#declare t_sabbia=t_sabbiosa;
#end

// This material arises from Christoph Hormann tutorial
// and was step by step modified following p.b.i. people
// indications (see credits above)
#declare m_water = color_map {
	[ 0.00 rgbt <0.217,0.552,0.877, 0.9> ]
	[ 0.90 rgbt <0.137,0.437,0.697, 0.95> ]
	[ 1.00 rgb <1,1,1> ]
}

#if(visible_water)
	#if(blurred_water)
		#declare BlurAmount = .75;// Amount of blurring  --- 0.2 works but .75 I like better
		#declare BlurSamples = blur_amount; //How many rays to shoot   --- 40 for my pc is impossible. 5 is not too bad
		#declare BlurTranslation=4; //Default 10 //4 carino
		
		#declare t_water = texture {
			average texture_map {
				#declare Ind = 0; 
		     	#declare S = seed(0); 
		      	#while(Ind < BlurSamples) 
		        	[1 // The pigment of the object: 
				    pigment{
				    	rgbt <0.217,0.552,0.877, 0.9>
				    }
		           // The surface finish: 
				    finish {
						diffuse 0.3
						ambient 0.0
				        reflection {0.3, 1 fresnel}
				        conserve_energy
						specular 0.3
						roughness 0.003
				    }
		           	// This is the actual trick:
		           	normal { 
		           		//bumps BlurAmount 
				        ripples BlurAmount
				        frequency 1
		             	translate <rand(S),rand(S),rand(S)>*BlurTranslation 
		             	scale 1000 
		             	//scale 0.001
		           	} 
		        	] 
		        	#declare Ind = Ind+1; 
		    	#end 
		    } 
		} 
	#else
	    /*
		#declare t_water = texture {
		    pigment{
		    	rgbt <0.217,0.552,0.877, 0.9>
		    	//gradient z 
		    	//color_map {m_water}
			    //turbulence 0.9
		    }
		    normal {
		        ripples 0.75
		        frequency 1
		        translate <-3,0,3>
		    }
		    finish {
				diffuse 0.3
				ambient 0.0
		        reflection {0.3, 1 fresnel}
		        conserve_energy
				specular 0.3
				roughness 0.003
		    }
		}
		*/
		#declare t_water = texture {
		    pigment{
		    	rgbt <0.217,0.552,0.877, 0.9>
		    }
		    normal {
		        ripples 0.25   // 0.75
		        frequency 0.1
		        translate <-10,0,10>
		    }
		    finish {
				diffuse 0.3
				ambient 0.0
		        reflection {0.3, 1 fresnel}
		        conserve_energy
				specular 0.3
				roughness 0.003
		    }
		}
	#end
#else
	#declare t_water = texture {
	    pigment{
	    	rgbt <1,1,1,1>
	    }
	}
#end

#declare i_water =interior {
    //ior 1.33
    ior 1.1 //1.03
    fade_distance 0.3
	fade_power 1000
    fade_color <0.217,0.552,0.877>
}

#declare m_acqua =material {
    texture {t_water}
    interior {i_water}
}

#declare t_torn_fish=texture{
	pigment {
		bozo 
        color_map {
           [ 0.0     rgb <1, 1, 1> ]
           [ 0.8     rgb <0.5, 0.5, 0.5> ]
           [ 1.0     rgb <0.9,0.9,0.9> ]
        }
		scale .1
	}
	finish{phong.4 ambient.3}
}

#declare t_gold=texture{
    T_Gold_2A
    normal {
        agate 0.75 
        scale 0.25  
        turbulence 0.2
    }
}



// Scene -------------------------------------------------------
// Sky ----------------------------------------------------
sky_sphere {
		pigment{
			bozo
			turbulence 0.76
			color_map { 
				[0.3  rgb<0.1,0.3,0.9> ]
				[0.5  rgbf<0.4,0.7,0.8,1> ]
				[0.6  rgb <0.3,0.6,0.9>     ]
				[0.65 rgb <0.4,0.7,0.99>     ]
				[1.0  rgb <0.2,0.5,0.8>     ] 
			}
		}
		pigment{
			color rgbf <0.2,0.8,0.9,0.6>
		}
}
	

// Floor --------------------------------------------------
#if (show_floor)
	plane { <0,1,0>, -1 
		texture { t_3}
	}
#end

// Moon ---------------------------------------------------
#if (show_moon)
	#declare c_moon=<7,8,10>;
	#declare moon=sphere {
		c_moon, 1
	}
	
	#if(light_from_moon)
		#declare luce_moon=light_source{
			<0,0,0>
			color rgb <1,0.98,0.76>
			looks_like{
				object{
					moon
					hollow
					material {
						m_moon_light
					}
				}
			}
		}
		light_source{luce_moon}
		// I need to hyde some stars...
		sphere {
			c_moon, 0.9
			texture {t_moon_light}
		}
	#else
		object{
			moon
			texture { pigment {rgb 1}	}
		}
	#end
#end


// Sand -----------------------------
#declare sand=isosurface {
	function {
		y-sin(x*3*pi)*0.1-f_noise3d(x,y,z)-sin(z*2*pi)*0.1*f_noise3d(0,0,z)
	}
	accuracy 0.01
  	contained_by {box {<-10,-0.6,-10>, < 25, 1.2, 25>}}
  	max_gradient 2.0
}

// Water ----------------------------
#declare water=isosurface {
	function {
		y-sin(x*2*pi)*0.03-f_noise3d(x,0,z)*0.3
	}
  	contained_by {box {<-10,-2,-10>, < 25, 1, 25>}}
	accuracy 0.01
}
//#declare water=box {<-10,-2,-10>, < 25, 1, 25>}

// Torn fish ----------------------
#declare torn_fish=union{
	lathe {cubic_spline 6,v-u,v,<.3,.5>,u/5,-v,-2*v rotate-90*z} 
	disc{0,z,.5 clipped_by{sphere{x,1}}}
	disc{0,z,.5 clipped_by{sphere{x,1}} translate -x}
	scale 0.5 // Ora il pesce  lungo 1
	rotate <0,90,0>
}
// --------------------------------
#declare pesce=object{
	torn_fish
	texture{t_torn_fish}
}


// IRTC -----------------------------
//#declare type_irtc=text { ttf "times.ttf", "IRTC", 1, 0 rotate <90, -30, 0> scale 6 translate <-4, -0.5, -3>} 
#declare rotate_irtc=<0, -30, 0>;
#declare translate_irtc=<-4, -0.5, -3>; 
#declare type_irtc=text { ttf "times.ttf", "IRTC", 1, 0 rotate 90*x scale 6} 
#if(show_irtc)
    object {type_irtc texture{pigment {color rgb 1}} rotate rotate_irtc translate translate_irtc}
#end


// Pebbles --------------------------
#if(show_bricks)
	#declare R1=seed(2);
	#declare R2=seed(41);
	#declare R3=seed(12);
	#declare l_box=0.1;
	
	/*
	#declare the_brick=union{
		// Home
		difference{
			box{0 l_box}
			// Interior
			box{0.1 l_box*0.9}
			// Door
			box{<l_box/3,0,-0.1> <2*l_box/3,l_box/2-l_box/10,2*l_box/3>}
			// Upper front and rear windows
			box{<l_box/10,l_box/2+l_box/10,-0.1> <l_box/4+l_box/10,l_box/2+l_box/4+l_box/10,l_box*1.1>}
			box{<l_box-l_box/10,l_box/2+l_box/10,-0.1> <l_box-l_box/4-l_box/10,l_box/2+l_box/4+l_box/10,l_box*1.1>}
			// Lower rear windows
			box{<l_box/10,l_box/10+l_box/10,l_box-0.1> <l_box/4+l_box/10,l_box/10+l_box/4+l_box/10,l_box-2*l_box/3>}
			box{<l_box-l_box/10,l_box/10+l_box/10,l_box-0.1> <l_box-l_box/4-l_box/10,l_box/10+l_box/4+l_box/10,l_box-2*l_box/3>}
			// Lateral upper windows
			box{<-0.1,l_box/2+l_box/10,l_box/10> <l_box*1.1,l_box/2+l_box/4+l_box/10,l_box/4+l_box/10>}
			box{<-0.1,l_box/2+l_box/10,l_box-l_box/10> <l_box*1.1,l_box/2+l_box/4+l_box/10,l_box-l_box/4-l_box/10>}
			// Lateral rear windows
			box{<-0.1,l_box/10,l_box/10> <l_box*1.1,l_box/10+l_box/4+l_box/10,l_box/4+l_box/10>}
			box{<-0.1,l_box/10+l_box/10,l_box-l_box/10> <l_box*1.1,l_box/10+l_box/4+l_box/10,l_box-l_box/4-l_box/10>}
			texture{pigment {color rgb 1}}
		}
		// 1st floor
		box{<0,l_box/2-l_box/10,0> <l_box,l_box/2,l_box> texture{pigment {color rgb 1}}}
		// Deck
		difference{
		    box {< 1,1,1>, <-1,0,-1>}
		    plane { x-y,  -sqrt(2)/2 }
		    plane { -x-y, -sqrt(2)/2 }
		    plane { z-y,  -sqrt(2)/2 }
		    plane { -z-y, -sqrt(2)/2 }
		    scale l_box*1.1/2
		    translate <0.5,1,0.5>*l_box
		    texture{pigment {color rgb x}}
		}
		// Chimney
		box {<l_box-l_box/10,l_box,l_box-l_box/10> <l_box-l_box/10-l_box/10,l_box+l_box/3,l_box-l_box/10-l_box/10> texture{pigment {color rgb 1}}}
		box {<l_box-l_box/10+l_box/30,l_box+l_box/3,l_box-l_box/10+l_box/30> <l_box-l_box/10-l_box/10-l_box/30,l_box+l_box/3+l_box/30,l_box-l_box/10-l_box/10+l_box/30> texture{pigment {color rgb 1}}}
	}
	*/
	//todo: cutoff of some pieces
	//#declare the_brick=box{0 l_box texture{t_gold}}
    // F_loemuller egg -------------------------
    //#include "shapes_lo.inc"
	#declare the_brick=object {Egg scale l_box texture{t_gold}}
	
	#declare Normal=<0,0,0>;
	#declare Normal_sand=<0,0,0>;
	#declare m_min=min_extent(type_irtc);
	#declare m_max=max_extent(type_irtc);
	
	#declare m_steps=100; //100
	#declare r_stop=0.90;
	
	#local i=0;
	union {
		#while (i<=m_steps)
			#local j=0;
			#local m_x=m_min.x+(m_max.x-m_min.x)*i/m_steps;
			#while (j<=m_steps)
				#local m_z=m_min.z+(m_max.z-m_min.z)*j/m_steps;
				#if(rand(R1)<=r_stop)
					#declare Intersection=trace(type_irtc, <m_x,10,m_z>, -y, Normal);
					#declare Intersection_sand=trace(sand, <m_x,10,m_z>, -y, Normal_sand);
					#if(vlength(Normal) > 0)
					    //#local intersec=Intersection;
					    /*
					    #if (vlength(Normal_sand) > 0)
					        #local Intersec=Intersection_sand;
					    #end
					    */
						#local correction=0.5+rand(R2)*2;
	    				#local Rotation=(rand(R3)-0.5)*40;
						//object{the_brick scale correction rotate Rotation translate <Intersection.x,-1,Intersection.z> texture{t_brick}}
						object{the_brick scale correction rotate Rotation translate <Intersection.x,Intersection_sand.y,Intersection.z> }
					#end
				#end
				#local j=j+1;
			#end
			#local i=i+1;
		#end
		rotate rotate_irtc
		translate translate_irtc
	}
#end

// Island ---------------------------
#if(show_island)
	object {sand
	  	texture {t_sabbia}
		rotate rotate_irtc
		translate translate_irtc
	}     
#end

#if(show_sea)
	plane { <0,1,0>, -1.7
		texture {t_sea}
	}
#end

#if(show_water)
	object {water
	  	material {m_acqua}
		#if(with_photons)
		  	photons {
			    target
			    reflection off
			    refraction on
			    collect off
		  	}
		#end
		translate <0,2,0>
	}
#end

#if(show_fishes)
	#if(three_big_fishes)
		//Tre Pescioni
		object {pesce scale 0.8 translate <0,2,0> rotate <0,30,0>}
		object {pesce scale 0.8 translate <0,2,-2> rotate <0,-20,0>}
		object {pesce scale 0.8 translate <0,2,-3> rotate <0,10,0>}
	#else
		/*
		object {pesce rotate <0,90,0>}
		sphere {<0,0,0> 0.5 pigment{rgbf <1,0,0,0.9>} translate <0,0,0>}
		*/
		#declare S1 = seed(123);
		#declare pz=5;
		#while(pz>-7)
			#declare pz=pz-12/fishes_number;
			#declare px=1+sin(2*pi*(pz/-3))-0.2+rand(S1)*0.4;
			#declare py=1.2+rand(S1)*0.8;
			object {pesce scale 0.3 rotate <0,rand(S1)*30,0> translate <px,py,pz>}			
		#end
	#end
#end